@@ -41,7 +41,7 @@ def register():
         View,
         ShowViewStart,
         ViewTreeWidth,
-        ViewTreeExpandedState,
+        ViewTreeState,
         ViewSearch,
         Icon,
         Property,
@@ -72,6 +72,7 @@ def register():
         ModuleConfigWizardItem,
         ModuleConfigWizardFirst,
         ModuleConfigWizardOther,
+        ModuleConfigWizardDone,
         ModuleInstallUpgradeStart,
         ModuleInstallUpgradeDone,
         Cache,
