@@ -188,7 +188,19 @@ if __name__ == '__main__':
 ''')
         time_locale = {}
         fp.write('TIME_LOCALE = \\\n')
-        for lang in ('cs_CZ', 'de_DE', 'en_US', 'es_ES', 'es_CO', 'fr_FR'):
+        for lang in [
+                'bg_BG',
+                'ca_ES',
+                'cs_CZ',
+                'de_DE',
+                'en_US',
+                'es_AR',
+                'es_ES',
+                'es_CO',
+                'fr_FR',
+                'nl_NL',
+                'ru_RU',
+                ]:
             time_locale[lang] = locale_strftime(lang)
             #fp.write('"' + lang + '": ' + repr(time_locale) + ',\n')
         #fp.write('}\n')
