@@ -3,8 +3,7 @@
 from trytond.protocols.sslsocket import SSLSocket
 from trytond.protocols.dispatcher import dispatch
 from trytond.config import CONFIG
-from trytond.protocols.common import daemon, GZipRequestHandlerMixin, \
-    RegisterHandlerMixin
+from trytond.protocols.common import daemon, RegisterHandlerMixin
 from trytond.exceptions import UserError, UserWarning, NotLogged, \
     ConcurrencyException
 import SimpleXMLRPCServer
@@ -38,11 +37,12 @@ def object_hook(dct):
     if '__class__' in dct:
         if dct['__class__'] == 'datetime':
             return datetime.datetime(dct['year'], dct['month'], dct['day'],
-                    dct['hour'], dct['minute'], dct['second'])
+                dct['hour'], dct['minute'], dct['second'], dct['microsecond'])
         elif dct['__class__'] == 'date':
             return datetime.date(dct['year'], dct['month'], dct['day'])
         elif dct['__class__'] == 'time':
-            return datetime.time(dct['hour'], dct['minute'], dct['second'])
+            return datetime.time(dct['hour'], dct['minute'], dct['second'],
+                dct['microsecond'])
         elif dct['__class__'] == 'buffer':
             return buffer(base64.decodestring(dct['base64']))
         elif dct['__class__'] == 'Decimal':
@@ -67,6 +67,7 @@ class JSONEncoder(json.JSONEncoder):
                         'hour': obj.hour,
                         'minute': obj.minute,
                         'second': obj.second,
+                        'microsecond': obj.microsecond,
                         }
             return {'__class__': 'date',
                     'year': obj.year,
@@ -78,6 +79,7 @@ class JSONEncoder(json.JSONEncoder):
                 'hour': obj.hour,
                 'minute': obj.minute,
                 'second': obj.second,
+                'microsecond': obj.microsecond,
                 }
         elif isinstance(obj, buffer):
             return {'__class__': 'buffer',
@@ -157,8 +159,7 @@ class GenericJSONRPCRequestHandler:
         return res
 
 
-class SimpleJSONRPCRequestHandler(GZipRequestHandlerMixin,
-        RegisterHandlerMixin,
+class SimpleJSONRPCRequestHandler(RegisterHandlerMixin,
         GenericJSONRPCRequestHandler,
         SimpleXMLRPCServer.SimpleXMLRPCRequestHandler,
         SimpleHTTPServer.SimpleHTTPRequestHandler):
@@ -305,34 +306,21 @@ class SimpleJSONRPCServer(SocketServer.TCPServer,
 
     def server_close(self):
         SocketServer.TCPServer.server_close(self)
-        for handler in self.handlers:
+        for handler in self.handlers.copy():
             self.shutdown_request(handler.request)
 
-    if sys.version_info[:2] <= (2, 6):
-
-        def shutdown_request(self, request):
-            """Called to shutdown and close an individual request."""
-            try:
-                #explicitly shutdown.  socket.close() merely releases
-                #the socket and waits for GC to perform the actual close.
-                request.shutdown(socket.SHUT_WR)
-            except socket.error:
-                pass  # some platforms may raise ENOTCONN here
-            self.close_request(request)
-
 
 class SimpleThreadedJSONRPCServer(SocketServer.ThreadingMixIn,
         SimpleJSONRPCServer):
     timeout = 1
     daemon_threads = True
+    disable_nagle_algorithm = True
 
     def server_bind(self):
         self.socket.setsockopt(socket.SOL_SOCKET,
                 socket.SO_REUSEADDR, 1)
         self.socket.setsockopt(socket.SOL_SOCKET,
             socket.SO_KEEPALIVE, 1)
-        self.socket.setsockopt(socket.IPPROTO_TCP,
-            socket.TCP_NODELAY, 1)
         SimpleJSONRPCServer.server_bind(self)
 
 
