@@ -1,9 +1,10 @@
 #This file is part of Tryton.  The COPYRIGHT file at the top level of
 #this repository contains the full copyright notices and license terms.
 from threading import local
+from sql import Flavor
+
 from trytond.tools.singleton import Singleton
-from trytond.backend import Database
-from trytond.config import CONFIG
+from trytond import backend
 
 
 class _TransactionManager(object):
@@ -57,6 +58,8 @@ class Transaction(local):
     __metaclass__ = Singleton
 
     cursor = None
+    database = None
+    close = None
     user = None
     context = None
     create_records = None
@@ -64,16 +67,28 @@ class Transaction(local):
     delete = None  # TODO check to merge with delete_records
     timestamp = None
 
-    def start(self, database_name, user, readonly=False, context=None):
+    def start(self, database_name, user, readonly=False, context=None,
+            close=False, autocommit=False):
         '''
         Start transaction
         '''
+        Database = backend.get('Database')
         assert self.user is None
+        assert self.database is None
         assert self.cursor is None
+        assert self.close is None
         assert self.context is None
-        database = Database(database_name).connect()
-        self.cursor = database.cursor(readonly=readonly)
+        if not database_name:
+            database = Database().connect()
+        else:
+            database = Database(database_name).connect()
+        Flavor.set(Database.flavor)
+        cursor = database.cursor(readonly=readonly,
+            autocommit=autocommit)
         self.user = user
+        self.database = database
+        self.cursor = cursor
+        self.close = close
         self.context = context or {}
         self.create_records = {}
         self.delete_records = {}
@@ -87,9 +102,11 @@ class Transaction(local):
         Stop transaction
         '''
         try:
-            self.cursor.close()
+            self.cursor.close(close=self.close)
         finally:
             self.cursor = None
+            self.database = None
+            self.close = None
             self.user = None
             self.context = None
             self.create_records = None
@@ -100,7 +117,8 @@ class Transaction(local):
     def set_context(self, context=None, **kwargs):
         if context is None:
             context = {}
-        manager = _AttributeManager(context=self.context.copy())
+        manager = _AttributeManager(context=self.context)
+        self.context = self.context.copy()
         self.context.update(context)
         if kwargs:
             self.context.update(kwargs)
@@ -112,10 +130,16 @@ class Transaction(local):
         return manager
 
     def set_user(self, user, set_context=False):
+        if user != 0 and set_context:
+            raise ValueError('set_context only allowed for root')
         manager = _AttributeManager(user=self.user,
-                context=self.context.copy())
+                context=self.context)
+        self.context = self.context.copy()
         if set_context:
-            self.context.update({'user': self.user})
+            if user != self.user:
+                self.context['user'] = self.user
+        else:
+            self.context.pop('user', None)
         self.user = user
         return manager
 
@@ -124,14 +148,19 @@ class Transaction(local):
         self.cursor = cursor
         return manager
 
-    def new_cursor(self):
+    def new_cursor(self, autocommit=False, readonly=False):
+        Database = backend.get('Database')
         manager = _CursorManager(self.cursor)
         database = Database(self.cursor.database_name).connect()
-        self.cursor = database.cursor()
+        self.cursor = database.cursor(autocommit=autocommit, readonly=readonly)
         return manager
 
     @property
     def language(self):
+        def get_language():
+            from trytond.pool import Pool
+            Config = Pool().get('ir.configuration')
+            return Config.get_language()
         if self.context:
-            return self.context.get('language') or CONFIG['language']
-        return CONFIG['language']
+            return self.context.get('language') or get_language()
+        return get_language()
